# -*- coding: UTF-8 -*-
# Copyright 2016-2024 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""This is the main module of Lino Prima.

.. autosummary::
   :toctree:

   lib
   projects


"""

__version__ = '25.1.0'

srcref_url = 'https://gitlab.com/synodalsoft/prima/blob/master/%s'
docs_url = "https://synodalsoft.gitlab.io/prima"
intersphinx_urls = dict(docs=docs_url)
doc_trees = ['docs']
