# -*- coding: UTF-8 -*-
# Copyright 2016-2024 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""This package contains the :term:`demo projects <demo project>` included with
Lino Prima.

.. autosummary::
   :toctree:

   prima1

"""
