# -*- coding: UTF-8 -*-
# generated by lino.sphinxcontrib.help_text_builder
# fmt: off
from django.utils.translation import gettext_lazy as _

help_texts = {
    'lino_prima.lib.prima.migrate.Migrator' : _("""The standard migrator for Lino Prima."""),
    'lino_prima.plugins.cert.Certificate.enrolment' : _("""The pupil for whom this certificate has been issued."""),
    'lino_prima.lib.prima.Subject' : _("""Django model user to represent a subject that can potentially be given at this school."""),
    'lino_prima.lib.prima.Subject.advanced' : _("""Whether this subject uses exams for evaluating the pupil’s progress."""),
    'lino_prima.lib.prima.Subject.icon_text' : _("""A single character to use as pictogram in the printed certificate. This is meant to contain a Unicode symbold or pictogram, e.g. from https://www.compart.com/en/unicode/block/U+1F300"""),
}
