# from django.conf import settings
# from lino.api import rt

def objects():
    # settings.SITE.site_config.update(
    #     default_build_method=rt.models.printing.BuildMethods.weasy2pdf)
    return []
